import { clsx } from 'clsx'

import {
  ShieldCheckIcon,
  UserIcon,
  UsersIcon
} from '@heroicons/react/20/solid'

import { Bars3Icon } from '@heroicons/react/24/outline'

import WebComponent from './WebComponent'
import { useState } from 'react'
import { ProfileSettings } from './ProfileSettings'
import { ChangePassword } from './ChangePassword'
import { UsersTable } from './UsersTable'
import { Fragment } from 'preact'
import { Transition } from '@headlessui/react'

export function SettingsWebComponent (props) {
  return (
    <WebComponent component='settings' {...props}>
      <Settings />
    </WebComponent>
  )
}

export function Settings ({ padding = '', modal }) {
  const [active, setActive] = useState(0)
  const [sidebarOpen, setSidebarOpen] = useState(false)

  const selectTab = (index) => {
    setActive(index)
    setSidebarOpen(false)
  }

  const openSidebar = () => {
    setSidebarOpen(true)
    modal?.hideCloseButton?.()
  }

  const closeSidebar = () => {
    setSidebarOpen(false)
    modal?.showCloseButton?.()
  }

  const navigation = [
    { name: 'Profile', icon: UserIcon },
    { name: 'Password', icon: ShieldCheckIcon },
    { name: 'Users', icon: UsersIcon }
  ]

  const views = [
    ProfileSettings,
    ChangePassword,
    UsersTable
  ]

  const View = views[active]

  const containerClass = clsx(
    '@container relative flex-1 flex flex-rows overflow-hidden',
    padding !== 'inner' && 'gap-4'
  )

  const sidebarClass = clsx(
    'grow-0 hidden @xl:flex flex-col w-[250px] border-r border-gray-200 pr-4',
    padding === 'inner' && 'p-4'
  )

  const viewClass = clsx(
    'grow',
    padding === 'inner' && 'p-4 py-6'
  )

  const sidebar = (
    <Transition show={sidebarOpen}>
      <Transition.Child
        as={Fragment}
        enter='transition-opacity ease-linear duration-300'
        enterFrom='opacity-0'
        enterTo='opacity-100'
        leave='transition-opacity ease-linear duration-300'
        leaveFrom='opacity-100'
        leaveTo='opacity-0'
      >
        <div className='fixed inset-0 z-20 bg-black/30' />
      </Transition.Child>

      <div className='fixed inset-0 z-30 flex' onClick={closeSidebar}>
        <Transition.Child
          as={Fragment}
          enter='transition ease-in-out duration-300 transform'
          enterFrom='-translate-x-full'
          enterTo='translate-x-0'
          leave='transition ease-in-out duration-300 transform'
          leaveFrom='translate-x-0'
          leaveTo='-translate-x-full'
        >
          <div className='z-30 bg-white shadow-lg w-[250px] p-4'>
            <Navigation
              active={active}
              setActive={selectTab}
              navigation={navigation}
            />
          </div>
        </Transition.Child>
      </div>
    </Transition>
  )

  return (
    <div className={containerClass}>
      <div className={sidebarClass}>
        <Navigation
          active={active}
          setActive={setActive}
          navigation={navigation}
        />
      </div>

      <div className={viewClass}>
        <BarsButton setSidebarOpen={openSidebar} />
        <View />
      </div>

      <div className='absolute'>
        {sidebar}
      </div>

    </div>
  )
}

export function Navigation ({ navigation, active, setActive }) {
  const current = (index) => {
    return index === active
  }

  return (
    <nav className='flex flex-1 flex-col'>
      <ul role='list' className='flex flex-1 flex-col'>
        <li>
          <ul role='list' className='space-y-1'>
            {navigation.map((item, index) => (
              <li key={item.name}>
                <a
                  href='#'
                  onClick={() => setActive(index)}
                  className={clsx(
                    current(index)
                      ? 'bg-gray-100 text-gray-900'
                      : 'text-gray-700 hover:text-gray-900 hover:bg-gray-50',
                    'group flex gap-x-3 rounded-lg p-2 px-3 text-sm leading-6'
                  )}
                >
                  <item.icon
                    className={clsx(
                      current(item) ? 'text-gray-900' : 'text-gray-400 group-hover:text-gray-900',
                      'h-5 w-5 shrink-0'
                    )}
                    aria-hidden='true'
                  />
                  {item.name}
                </a>
              </li>
            ))}
          </ul>
        </li>

      </ul>
    </nav>
  )
}

function BarsButton ({ setSidebarOpen }) {
  return (
    <button type='button' className='-m-2.5 -mt-4 p-2.5 text-gray-400 @xl:hidden' onClick={() => setSidebarOpen(true)}>
      <span className='sr-only'>Open sidebar</span>
      <Bars3Icon className='h-6 w-6' aria-hidden='true' />
    </button>
  )
}
