import { EllipsisVerticalIcon } from '@heroicons/react/20/solid'
import { clsx } from 'clsx'
import { EditRole, useRoleModal } from './modals/EditRole'
import { InviteUser, useInviteModal } from './modals/InviteUser'
import WebComponent from './WebComponent'
import {
  Dropdown,
  Role,
  useCurrentUser,
  useUsers,
  useDeleteUser,
  LightTable,
  Header,
  Avatar
} from '@userspace-auth/core'

export function UsersTableWebComponent (props) {
  return (
    <WebComponent component='users-table' {...props}>
      <UsersTable />
    </WebComponent>
  )
}

export function UsersTable (props) {
  const inviteModal = useInviteModal()
  const roleModal = useRoleModal()

  const { users } = useUsers()
  const { currentUser } = useCurrentUser()
  const { openModal } = inviteModal

  return (
    <div>
      <Header
        title='Users'
        description='Manage users in your network and their permissions'
        action='Invite user'
        actionVisible={currentUser.hasMinRole(Role.Admin)}
        onAction={openModal}
      />
      <div className='mt-8'>
        <Table users={users} currentUser={currentUser} roleModal={roleModal} />
      </div>
      {/* modals */}
      <InviteUser modal={inviteModal} />
      <EditRole modal={roleModal} />
    </div>
  )
}

const Table = ({ users, currentUser, roleModal }) => {
  const columns = [
    {
      name: 'Name',
      render: user => <UserDetails user={user} />
    },
    {
      name: 'Status',
      render: user => <Status status={user.status} />
    },
    {
      name: 'Role',
      render: user => user.role
    },
    {
      name: '',
      width: 'w-12',
      render: user => <Actions person={user} currentUser={currentUser} roleModal={roleModal} />
    }
  ]

  return (
    <LightTable columns={columns} data={users} />
  )
}

const UserDetails = ({ user }) => {
  return (
    <div className='flex items-center'>
      <div className='h-11 w-11 shrink-0'>
        <Avatar className='h-11 w-11 rounded-full' profile={user} />
      </div>
      <div className='ml-4'>
        <div className='font-medium text-gray-900'>{user.name}</div>
        <div className='text-gray-500'>{user.email}</div>
      </div>
    </div>
  )
}

const Status = ({ status }) => {
  const className = clsx(
    'inline-flex items-center rounded-md px-2 py-1 text-xs font-medium ring-1 ring-inset',
    { 'bg-green-50 text-green-700 ring-green-600/20': status === 'Active' },
    { 'bg-gray-50 text-gray-500 ring-gray-600/20': status === 'Inactive' },
    { 'bg-indigo-50 text-indigo-700 ring-indigo-600/20': status === 'Invited' },
    { 'bg-red-50 text-red-500 ring-red-600/20': status === 'Blocked' },
    { 'bg-amber-50 text-amber-600 ring-amber-700/20': status === 'WaitingVerification' }
  )

  const statusNames = {
    Active: 'Active',
    Inactive: 'Inactive',
    Invited: 'Invited',
    Blocked: 'Blocked',
    WaitingVerification: 'Waiting verification'
  }

  return (
    <span className={className}>
      {statusNames[status] || 'Unknown'}
    </span>
  )
}

const Actions = ({ currentUser, person, roleModal }) => {
  const { Menu, MenuItem } = Dropdown

  const canDelete = currentUser.hasMinRole(Role.Admin) && currentUser.id !== person.id
  const hasAccess = currentUser.hasMinRole(Role.Admin) && currentUser.id !== person.id && currentUser.hasHigherRole(person)

  const { deleteUser } = useDeleteUser()

  const handleDeleteUser = (e) => {
    if (canDelete) {
      deleteUser({ userId: person.id })
    }
  }

  const handleChangeRole = (e) => {
    roleModal.openRoleModal({ userId: person.id, role: person.role })
  }

  if (!hasAccess) {
    return null
  }

  return (
    <Menu icon={EllipsisVerticalIcon} placement='bottom-end'>
      <MenuItem label='Edit role' onClick={handleChangeRole} />
      <MenuItem label='Delete user' danger onClick={handleDeleteUser} />
    </Menu>
  )
}
